@@ -24,7 +24,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <assert.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <npth.h>
@@ -46,7 +45,6 @@ struct trustitem_s
                              constraints. */
     int cm:1;             /* Use chain model for validation. */
     int qual:1;           /* Root CA for qualified signatures.  */
-    int de_vs:1;          /* Root CA for de-vs compliant PKI.    */
   } flags;
   unsigned char fpr[20];  /* The binary fingerprint. */
 };
@@ -130,24 +128,6 @@ clear_trusttable (void)
 }
 
 
-/* Return the name of the system trustlist.  Caller must free.  */
-static char *
-make_sys_trustlist_name (void)
-{
-  if (opt.sys_trustlist_name
-      && (strchr (opt.sys_trustlist_name, '/')
-          || strchr (opt.sys_trustlist_name, '\\')
-          || (*opt.sys_trustlist_name == '~'
-              && opt.sys_trustlist_name[1] == '/')))
-    return make_absfilename (opt.sys_trustlist_name, NULL);
-  else
-    return make_filename (gnupg_sysconfdir (),
-                          (opt.sys_trustlist_name ?
-                           opt.sys_trustlist_name : "trustlist.txt"),
-                          NULL);
-}
-
-
 static gpg_error_t
 read_one_trustfile (const char *fname, int systrust,
                     trustitem_t **addr_of_table,
@@ -216,7 +196,7 @@ read_one_trustfile (const char *fname, int systrust,
             }
           /* fixme: Should check for trailing garbage.  */
 
-          etcname = make_sys_trustlist_name ();
+          etcname = make_filename (gnupg_sysconfdir (), "trustlist.txt", NULL);
           if ( !strcmp (etcname, fname) ) /* Same file. */
             log_info (_("statement \"%s\" ignored in '%s', line %d\n"),
                       "include-default", fname, lnr);
@@ -326,8 +306,6 @@ read_one_trustfile (const char *fname, int systrust,
             ti->flags.cm = 1;
           else if (n == 4 && !memcmp (p, "qual", 4) && systrust)
             ti->flags.qual = 1;
-          else if (n == 5 && !memcmp (p, "de-vs", 5) && systrust)
-            ti->flags.de_vs = 1;
           else
             log_error ("flag '%.*s' in '%s', line %d ignored\n",
                        n, p, fname, lnr);
@@ -370,24 +348,17 @@ read_trustfiles (void)
     return gpg_error_from_syserror ();
   tableidx = 0;
 
-  if (opt.no_user_trustlist)
-    fname = NULL;
-  else
+  fname = make_filename_try (gnupg_homedir (), "trustlist.txt", NULL);
+  if (!fname)
     {
-      fname = make_filename_try (gnupg_homedir (), "trustlist.txt", NULL);
-      if (!fname)
-        {
-          err = gpg_error_from_syserror ();
-          xfree (table);
-          return err;
-        }
+      err = gpg_error_from_syserror ();
+      xfree (table);
+      return err;
     }
 
-  if (!fname || (ec = gnupg_access (fname, F_OK)))
+  if ((ec = gnupg_access (fname, F_OK)))
     {
-      if (!fname)
-        ; /* --no-user-trustlist active.  */
-      else if ( ec == GPG_ERR_ENOENT )
+      if ( ec == GPG_ERR_ENOENT )
         ; /* Silently ignore a non-existing trustfile.  */
       else
         {
@@ -395,7 +366,7 @@ read_trustfiles (void)
           log_error (_("error opening '%s': %s\n"), fname, gpg_strerror (err));
         }
       xfree (fname);
-      fname = make_sys_trustlist_name ();
+      fname = make_filename (gnupg_sysconfdir (), "trustlist.txt", NULL);
       systrust = 1;
     }
   err = read_one_trustfile (fname, systrust, &table, &tablesize, &tableidx);
@@ -431,13 +402,10 @@ read_trustfiles (void)
 
 
 /* Check whether the given fpr is in our trustdb.  We expect FPR to be
- * an all uppercase hexstring of 40 characters.  If ALREADY_LOCKED is
- * true the function assumes that the trusttable is already locked.
- * If LISTMODE is set, a status line TRUSTLISTFPR is emitted first and
- * disabled keys are not listed.
- */
+   an all uppercase hexstring of 40 characters.  If ALREADY_LOCKED is
+   true the function assumes that the trusttable is already locked. */
 static gpg_error_t
-istrusted_internal (ctrl_t ctrl, const char *fpr, int listmode, int *r_disabled,
+istrusted_internal (ctrl_t ctrl, const char *fpr, int *r_disabled,
                     int already_locked)
 {
   gpg_error_t err = 0;
@@ -476,8 +444,6 @@ istrusted_internal (ctrl_t ctrl, const char *fpr, int listmode, int *r_disabled,
       for (ti=trusttable, len = trusttablesize; len; ti++, len--)
         if (!memcmp (ti->fpr, fprbin, 20))
           {
-            if (listmode && ti->flags.disabled)
-              continue;
             if (ti->flags.disabled && r_disabled)
               *r_disabled = 1;
 
@@ -485,26 +451,17 @@ istrusted_internal (ctrl_t ctrl, const char *fpr, int listmode, int *r_disabled,
                in a locked state.  */
             if (already_locked)
               ;
-            else if (ti->flags.relax || ti->flags.cm || ti->flags.qual
-                     || ti->flags.de_vs)
+            else if (ti->flags.relax || ti->flags.cm || ti->flags.qual)
               {
                 unlock_trusttable ();
                 locked = 0;
                 err = 0;
-                if (listmode)
-                  {
-                    char hexfpr[2*20+1];
-                    bin2hex (ti->fpr, 20, hexfpr);
-                    err = agent_write_status (ctrl,"TRUSTLISTFPR", hexfpr,NULL);
-                  }
-                if (!err && ti->flags.relax)
+                if (ti->flags.relax)
                   err = agent_write_status (ctrl,"TRUSTLISTFLAG", "relax",NULL);
                 if (!err && ti->flags.cm)
                   err = agent_write_status (ctrl,"TRUSTLISTFLAG", "cm", NULL);
                 if (!err && ti->flags.qual)
                   err = agent_write_status (ctrl,"TRUSTLISTFLAG", "qual",NULL);
-                if (!err && ti->flags.de_vs)
-                  err = agent_write_status (ctrl,"TRUSTLISTFLAG", "de-vs",NULL);
               }
 
             if (!err)
@@ -526,24 +483,20 @@ istrusted_internal (ctrl_t ctrl, const char *fpr, int listmode, int *r_disabled,
 gpg_error_t
 agent_istrusted (ctrl_t ctrl, const char *fpr, int *r_disabled)
 {
-  return istrusted_internal (ctrl, fpr, 0, r_disabled, 0);
+  return istrusted_internal (ctrl, fpr, r_disabled, 0);
 }
 
 
 /* Write all trust entries to FP. */
 gpg_error_t
-agent_listtrusted (ctrl_t ctrl, void *assuan_context, int status_mode)
+agent_listtrusted (void *assuan_context)
 {
   trustitem_t *ti;
   char key[51];
-  int table_locked;
   gpg_error_t err;
   size_t len;
-  strlist_t allhexgrips = NULL;
-  strlist_t sl;
 
   lock_trusttable ();
-  table_locked = 1;
   if (!trusttable)
     {
       err = read_trustfiles ();
@@ -555,7 +508,6 @@ agent_listtrusted (ctrl_t ctrl, void *assuan_context, int status_mode)
         }
     }
 
-  err = 0;
   if (trusttable)
     {
       for (ti=trusttable, len = trusttablesize; len; ti++, len--)
@@ -563,15 +515,6 @@ agent_listtrusted (ctrl_t ctrl, void *assuan_context, int status_mode)
           if (ti->flags.disabled)
             continue;
           bin2hex (ti->fpr, 20, key);
-          if (status_mode)
-            {
-              if (!add_to_strlist_try (&allhexgrips, key))
-                {
-                  err = gpg_error_from_syserror ();
-                  goto leave;
-                }
-              continue;
-            }
           key[40] = ' ';
           key[41] = ((ti->flags.for_smime && ti->flags.for_pgp)? '*'
                      : ti->flags.for_smime? 'S': ti->flags.for_pgp? 'P':' ');
@@ -581,24 +524,8 @@ agent_listtrusted (ctrl_t ctrl, void *assuan_context, int status_mode)
         }
     }
 
-  if (status_mode)
-    {
-      unlock_trusttable ();
-      table_locked = 0;
-
-      /* The back and forth converting of the fingerprint and all the
-       * locking and unlocking is somewhat clumsy but helps to re-use
-       * existing code.  */
-      for (sl = allhexgrips; sl; sl = sl->next)
-        if ((err = istrusted_internal (ctrl, sl->d, 1, NULL, 0)))
-          goto leave;
-    }
-
- leave:
-  if (table_locked)
-    unlock_trusttable ();
-  free_strlist (allhexgrips);
-  return err;
+  unlock_trusttable ();
+  return 0;
 }
 
 
@@ -626,7 +553,7 @@ insert_colons (const char *string)
         }
     }
   *p = 0;
-  assert (strlen (buffer) <= nnew);
+  log_assert (strlen (buffer) <= nnew);
 
   return buffer;
 }
@@ -696,7 +623,7 @@ agent_marktrusted (ctrl_t ctrl, const char *name, const char *fpr, int flag)
   if (!fname)
     return gpg_error_from_syserror ();
 
-  if ((ec = gnupg_access (fname, W_OK)) && ec != GPG_ERR_ENOENT)
+  if ((ec = access (fname, W_OK)) && ec != GPG_ERR_ENOENT)
     {
       xfree (fname);
       return gpg_error (GPG_ERR_EPERM);
@@ -812,7 +739,7 @@ agent_marktrusted (ctrl_t ctrl, const char *name, const char *fpr, int flag)
      sure that nobody else plays with our file and force a reread.  */
   lock_trusttable ();
   clear_trusttable ();
-  if (!istrusted_internal (ctrl, fpr, 0, &is_disabled, 1) || is_disabled)
+  if (!istrusted_internal (ctrl, fpr, &is_disabled, 1) || is_disabled)
     {
       unlock_trusttable ();
       xfree (fprformatted);
